//    This is an extension for the firefox browser for tracking stocks
//    Copyright (C) 2008  Prassnitha Sampath
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

var StockList = {

    // Following are the global variables used in this program
    //
    // Dynamic array of stocks and info
    //   stockListLen - number of stocks I am tracking. Note that some of these may be deleted stocks
    //   stockSymList - dynamic array of stocks I am tracking
    //   stockPriceList - dynamic array of prices of stocks I am tracking
    //   stockChangeList - dynamic array of prices of stocks I am tracking
    //   stockLowList - dynamic array of prices of stocks I am tracking
    //   stockHighList - dynamic array of prices of stocks I am tracking
    //   stockActive - whether this stock is being tracked or not (could be deleted by the user)
    //
    // File corresponding to data. Read when browser is open. Written when browser is closed.
    //   dataFile - List of symbols that the stock is tracking. Planning to store this in the profile directory
    //
    //
    // Updating related control
    //   autoscroll
    //   timeoutScroll
    //   timeoutUpdate - timeout between scrolls
    //   timeoutID - timeoutID of the timeout, this can be used to terminate wakeup events
    //

    onLoad: function() {
	this.initialized = true;

	// Global variables for stock information
	stockListLen = 0;
	stockSymList = new Array();
	stockPriceList = new Array();
	stockChangeList = new Array();
	stockLowList = new Array();
	stockHighList = new Array();
	stockActive = new Array();


	timeoutUpdate = 10000;

	lastStockIdClicked = -1;

	/* Open a file in the profile folder where we store
	   the stocks we want to track */
	dataFile = Components.classes["@mozilla.org/file/directory_service;1"]
	   .getService(Components.interfaces.nsIProperties)
	   .get("ProfD", Components.interfaces.nsIFile);
	dataFile.append("Stocktrax.txt");
	// Now, file is initialized with the right file name and full path
	
	this.readDataFile();
	this.getAllPrices();
    },

    readDataFile: function() {
	var istream = Components.classes["@mozilla.org/network/file-input-stream;1"]
	  .createInstance(Components.interfaces.nsIFileInputStream);
	istream.init(dataFile, 0x01, 0444, 0);
	
	istream.QueryInterface(Components.interfaces.nsILineInputStream);

	var stock = {};
	var validRead;

	validRead = istream.readLine(stock);
	if (stock.value == "true") {
	    autoscroll = true;
	} else {
	    autoscroll = false;
	}

	var button = window.document.getElementById("ScrollCtrl");
	if (autoscroll) {
	    button.label = "AutoScroll:Off";
	} else {
	    button.label = "AutoScroll:On";
	}

	validRead = istream.readLine(stock);
	// Have to convert this into a number
	//	timeoutScroll = 5000;
	timeoutScroll = parseInt(stock.value);

	var scrollspeedtxt = window.document.getElementById("ScrollSpeedTxt");
	scrollspeedtxt.value = timeoutScroll;

	// read lines into array
	do {
	    validRead = istream.readLine(stock);
	    if (this.checkSymbol(stock.value)) {
		stockSymList.push(stock.value);
		stockPriceList.push(0.0);
		stockChangeList.push(0.0);
		stockLowList.push(0.0);
		stockHighList.push(0.0);
		stockActive.push(true);
		stockListLen++;
	    }
	} while (validRead);

	istream.close();
    },

    writeDataFile: function() {
	var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
	   .createInstance(Components.interfaces.nsIFileOutputStream);
	
	// use 0x02 | 0x10 to open file for appending.
	foStream.init(dataFile, -1, 0644, 0); 
	// write, create, truncate
	// In a c file operation, we have no need to set file mode with or operation,
	// directly using "r" or "w" usually.
	var i = 0;
	var str;

	if (autoscroll) {
	    str = "true\n";
	} else {
	    str = "false\n";
	}
	foStream.write(str, str.length);

	str = timeoutScroll + "\n";
	foStream.write(str, str.length);
	
	while (i < stockListLen) {
	    if (stockActive[i]) {
		str = stockSymList[i] + "\n";
		foStream.write(str, str.length);
	    }
	    i++;
	}
	foStream.close();
    },

    onUnload: function() {
	clearTimeout(timeoutID);
    },
    
    checkSymbol: function(symbol) {
	var ios = Components.classes["@mozilla.org/network/io-service;1"]
                    .getService(Components.interfaces.nsIIOService);
	var offlineMode = ios.offline;

	if (offlineMode) {
	    return true;
	}

	var valid = false;
	var httpRequest = null;
	var fullUrl = "http://quote.yahoo.com/d/quotes.csv?f=sl1d1t1c1ohgv&e=.csv&s=" + symbol;
	httpRequest = new XMLHttpRequest();
	
	httpRequest.open("GET", fullUrl, false);
	httpRequest.send(null);
	var output = httpRequest.responseText;
	
	if (output.length) {
	    output = output.replace(/\W*$/, "");
	    var fieldArray = output.split(","); 
	    if (parseFloat(fieldArray[1]) > 0) {
		valid = true;
	    }
	}
	return valid;
	    
    },
    
    getStockPrice: function(i){

	var ios = Components.classes["@mozilla.org/network/io-service;1"]
                    .getService(Components.interfaces.nsIIOService);
	var offlineMode = ios.offline;
	if (offlineMode) {
	    stockPriceList[i] = " - ";
	    stockChangeList[i] =  " - ";
	    stockLowList[i] = " - ";
	    stockHighList[i] = " - ";
	    return;
	}

	var symbol = stockSymList[i];
	var httpRequest = null;
	var fullUrl = "http://quote.yahoo.com/d/quotes.csv?f=sl1d1t1c1ohgv&e=.csv&s=" + symbol;
	httpRequest = new XMLHttpRequest();
	
	httpRequest.open("GET", fullUrl, false);
	httpRequest.send(null);
	
	var output = httpRequest.responseText;
	
	if (output.length) {
	    output = output.replace(/\W*$/, "");
	    
	    var fieldArray = output.split(","); 
	    stockPriceList[i] = fieldArray[1];
	    stockChangeList[i] = fieldArray[4];
	    stockLowList[i] = fieldArray[6];
	    stockHighList[i] = fieldArray[7];
	}
    },
    
    getAllPrices: function() {
 	var i = 0;
	while (i < stockListLen) {
	    this.getStockPrice(i);
	    this.updateDisplay(i);
	    i++;
	}
	timeoutID = setTimeout("StockList.getAllPrices()", timeoutUpdate);
    },

    updateDisplay: function(i) {
	var symbol = stockSymList[i];
	var symbolPrice = stockPriceList[i];
	var symbolChange = stockChangeList[i];
	var symbolLow = stockLowList[i];
	var symbolHigh = stockHighList[i];

	
	var listitem = window.document.getElementById(symbol);
	if (listitem != null && stockActive[i]) {
	    var listentry;
	    listentry = window.document.getElementById(symbol+"Price");
	    listentry.label = stockPriceList[i];
	    listentry = window.document.getElementById(symbol+"Change");
	    listentry.label = stockChangeList[i];
	    listentry = window.document.getElementById(symbol+"Low");
	    listentry.label = stockLowList[i];
	    listentry = window.document.getElementById(symbol+"High");
	    listentry.label = stockHighList[i];
	} else if ( listitem != null && ! stockActive[i]) {
	    var list = window.document.getElementById("ListOfStocks");
	    list.removeChild(listitem);
	} else if (stockActive[i]){
	    var list;
	    list = window.document.getElementById("ListOfStocks");

	    listitem = window.document.createElement("listitem");
	    listitem.setAttribute("id", stockSymList[i]);
	    listitem.setAttribute("ondblclick", "StockList.followLink(" + i + ")");
	    listitem.setAttribute("onclick", "StockList.saveSymbol(" + i + ")");

	    var listentry;
	    listentry = window.document.createElement("listcell");
	    listentry.setAttribute("label", stockSymList[i]);
	    listentry.setAttribute("id", stockSymList[i] + "Name");
	    listitem.appendChild(listentry);

	    listentry = window.document.createElement("listcell");
	    listentry.setAttribute("label", stockPriceList[i]);
	    listentry.setAttribute("id", stockSymList[i] + "Price");
	    listitem.appendChild(listentry);

	    listentry = window.document.createElement("listcell");
	    listentry.setAttribute("label", stockChangeList[i]);
	    listentry.setAttribute("id", stockSymList[i] + "Change");
	    listitem.appendChild(listentry);

	    listentry = window.document.createElement("listcell");
	    listentry.setAttribute("label", stockLowList[i]);
	    listentry.setAttribute("id", stockSymList[i] + "Low");
	    listitem.appendChild(listentry);

	    listentry = window.document.createElement("listcell");
	    listentry.setAttribute("label", stockHighList[i]);
	    listentry.setAttribute("id", stockSymList[i] + "High");
	    listitem.appendChild(listentry);

	    list.appendChild(listitem);

	}
    },

    saveSymbol: function(i) {
	lastStockIdClicked = i;
    },

    deleteStock: function() {
	if (lastStockIdClicked >=0) {
	    stockActive[lastStockIdClicked] = false;
	    clearTimeout(timeoutID);
	    this.getAllPrices();
	    this.writeDataFile();
	    this.informAllWindows();
	}
    },

    addStock: function() {
	var symtxt = window.document.getElementById("StockSymNew").value;
	symtxt = symtxt.toUpperCase();
	var found = false;
	var valid = false;
	var new_stock = true;

	var i = 0;
	while (i < stockListLen) {
	    if (symtxt == stockSymList[i]) {
		if (stockActive[i])
		    found = true;
		new_stock = false;
		stockActive[i] = true;
	    }
	    i++;
	}
	
	valid = this.checkSymbol(symtxt);
	if (!valid) {
	    alert("Symbol " + symtxt + " is invalid");
	}
	if (found) {
	    alert("Symbol " + symtxt + " is already being tracked");
	}

	if (valid && new_stock) {
	    stockSymList.push(symtxt);
	    stockPriceList.push(0.0);
	    stockChangeList.push(0.0);
	    stockLowList.push(0.0);
	    stockHighList.push(0.0);
	    stockActive.push(true);
	    stockListLen++;
	}
	clearTimeout(timeoutID);
	this.getAllPrices();
	this.writeDataFile();
	this.informAllWindows();
	window.document.getElementById("StockSymNew").value = "";
    },
    
    toggleAutoScroll: function() {
	autoscroll = !autoscroll;
	var button = window.document.getElementById("ScrollCtrl");
	if (autoscroll) {
	    button.label = "AutoScroll:Off";
	} else {
	    button.label = "AutoScroll:On";
	}
	this.writeDataFile();
	this.informAllWindows();
    },

    updateScrollSpeed: function() {
	var tbox = window.document.getElementById("ScrollSpeedTxt");
	var new_num = parseInt(tbox.value);
	if (!isNaN(new_num)) {
	    timeoutScroll = new_num;
	    tbox.value = timeoutScroll;
	    this.writeDataFile();
	    this.informAllWindows();
	} else {
	    tbox.value = timeoutScroll;
	    alert("Timeout should be a number in millisec");
	}
    },

    followLink: function(i) {
	var url = "http://finance.yahoo.com/q?s=" + stockSymList[i];
	window.open(url);
    },

    informAllWindows: function() {

	var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                   .getService(Components.interfaces.nsIWindowMediator);
	var enumerator = wm.getEnumerator(null);
	while(enumerator.hasMoreElements()) {
	    var win = enumerator.getNext();
	    win.QueryInterface(Components.interfaces.nsIDOMWindow);
	    var over = win.document.getElementById('stockTraxStatusBar');
	    if (over) {
		win.Stocktrax.readDataFile();
	    }
	}
    },

};


/* Registering the service to startup when windows starts up */
window.addEventListener("load", function(e) { StockList.onLoad(); }, false); 

/* Removing the service when windows ends */
window.addEventListener("unload", function(e) { StockList.onUnload(); }, false); 